part of hakkit.hakk.builder.basic;

class _ExpressionNode extends _OperatorClass
{
  final leftSide;
  final rightSide;
  final String operation;
  final _HakkObject memLoc;
  var callParams;
  String functionName;
  _ExpressionNode(this.leftSide, this.rightSide, this.operation) : memLoc = _MMU.malloc('expression${_getContextLabel()}');
  _ExpressionNode.call(this.functionName, this.callParams) : leftSide = val(0), rightSide = val(0), operation = 'call', memLoc = _MMU.malloc('expression${_getContextLabel()}');
  
  static String _getContextLabel(){
    return _globalContext.isEmpty ? '' : _globalContext.last;
  }
  
  void compile(){
    _expressionQueue.forEach((_ExpressionNode n){
      _HakkObject.addLine('$n');
    });
       
    _expressionQueue.clear();
  }
  
  @override toString(){
   final ls = leftSide is _HakkObject ? '$leftSide' : '${leftSide.memLoc}';
   final rs = rightSide is _HakkObject ? '$rightSide' : '${rightSide.memLoc}';
   
   switch(operation){
     case 'turnl':
        return 'turn: [${memLoc}, 1]';
     case 'turnr':
        return 'turn: [${memLoc}, 0]';
     case 'movef':
     case 'moveb':
        return '${operation}: ${memLoc}';
     case 'call':
       if (callParams != null){
         callParams.forEach((p){
           if (p is _ExpressionNode && p.operation == 'call'){
             throw 'passing a GOSUB as a parameter is currently not supported.';
           }
           push(p);
         });
       }
       final sb = new StringBuffer();
       sb.write('call: ${functionName}${_CR}');
       sb.write('    - pop: ${memLoc}');
       return sb.toString();
     case 'pop':
      return 'pop: ${memLoc}';
     case 'pow':
     case 'add':
     case 'mul':
     case 'div':
     case 'sub':
      return '${operation}: [${memLoc}, ${ls}, ${rs}]';
     case 'rnd':
     case 'sin':
     case 'cos':
       return '${operation}: [${memLoc}, ${ls}]';
     case 'jl':
     case 'jg':
     case 'jle':
     case 'jge':
     case 'je':
       final li = _nextLabelID;
       final sb = new StringBuffer();
       sb.write('${operation}: [${ls}, ${rs}, __istrue_${li}__]${_CR}');
       sb.write('    - set: [${memLoc}, false]${_CR}');
       sb.write('    - goto: __logic_finish_${li}__${_CR}');
       sb.write('    - lbl: __istrue_${li}__${_CR}');
       sb.write('    - set: [${memLoc}, true]${_CR}');
       sb.write('    - lbl: __logic_finish_${li}__');
       return sb.toString();
     case '&':
       final andFinish = _nextLabelID;
       int li = _nextLabelID;
       final sb = new StringBuffer();
       sb.write('je: [${ls}, true, __istrue_${li}__]${_CR}');
       sb.write('    - set: [${memLoc}, false]${_CR}');
       sb.write('    - goto: __and_finish_${andFinish}__${_CR}');
       sb.write('    - lbl: __istrue_${li}__${_CR}');
       li = _nextLabelID;
       sb.write('    - je: [${rs}, true, __istrue_${li}__]${_CR}');
       sb.write('    - set: [${memLoc}, false]${_CR}');
       sb.write('    - goto: __and_finish_${andFinish}__${_CR}');
       sb.write('    - lbl: __istrue_${li}__${_CR}');    
       sb.write('    - set: [${memLoc}, true]${_CR}');
       sb.write('    - lbl: __and_finish_${andFinish}__');
       return sb.toString();
     case '|':
       final li = _nextLabelID;
       final sb = new StringBuffer();
       sb.write('je: [${ls}, true, __istrue_${li}__]${_CR}');
       sb.write('    - set: [${memLoc}, false]${_CR}');
       sb.write('    - goto: __or_isfalse_${li}__${_CR}');
       sb.write('    - lbl: __istrue_${li}__${_CR}');
       sb.write('    - set: [${memLoc}, true]${_CR}');
       sb.write('    - goto: __or_finish_${li}__${_CR}');
       sb.write('    - lbl: __or_isfalse_${li}__${_CR}');
       sb.write('    - je: [${rs}, true, __istrue_${li}__]${_CR}');
       sb.write('    - set: [${memLoc}, false]${_CR}');
       sb.write('    - lbl: __or_finish_${li}__');
       return sb.toString();
   }
  }
}