part of hakkit.hakk.builder.test;

void advanced_tests(){
  group('advanced -', (){
    test('valid minimum', (){
        doit() =>
          hakk('test', (){
            blocks();
            code((){
              codeBlock('core', (){
                I.exit(0);
              });
            });
          });

        expect(doit().startsWith('name: "test"'), isTrue);
      });

      test('missing body', (){
        doit() => hakk('test', (){});
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('code before blocks throws', (){
        doit() => hakk('test', (){
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
          });
          blocks();
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('Instruction in blocks section throws', (){
        doit() => hakk('test', (){
          blocks((){
            I.nop();
          });
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('hakk called twice throws', (){
        doit() => hakk('test', (){
          blocks((){
            hakk('foo', (){});
          });
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('block in code block throws', (){
        doit() => hakk('test', (){
          blocks();
          code((){
            codeBlock('core', (){
              B.instruction('foo', 1, 1);
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('nested code blocks throws', (){
        doit() => hakk('test', (){
          blocks();
          code((){
            codeBlock('core', (){
              codeBlock('foo', (){});
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('nested code throws', (){
        doit() => hakk('test', (){
          blocks();
          code((){
            code((){});
            codeBlock('core', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('blocks section twice throws', (){
        doit() => hakk('test', (){
          blocks();
          blocks();
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('code section twice throws', (){
        doit() => hakk('test', (){
          blocks();
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
          });
          code((){});
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('codeBlock at top level throws', (){
        doit() => hakk('test', (){
          blocks();

          codeBlock('core', (){
            I.exit(0);
          });

          code((){
            codeBlock('core', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('no code blocks throws', (){
        doit() => hakk('test', (){
          blocks();
          code((){
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('SHP with no blocks defined throws', (){
        doit() => hakk('test', (){
          blocks((){
            B.memory('mem1', 0, 2);
            B.instruction('inst1', 0, 3);
            B.subHakkPort('shp1', 0, 4, 'core', 'sh1', (){

            });
          });
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('Instruction in SHP throws', (){
        doit() => hakk('test', (){
          blocks((){
            B.memory('mem1', 0, 2);
            B.instruction('inst1', 0, 3);
            B.subHakkPort('shp1', 0, 4, 'inst1', 'sh1', (){
              B.shpBlock('mem1', 0, 1);
              I.nop();
            });
          });
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
            codeBlock('inst1', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('non-shp block in SHP throws', (){
        doit() => hakk('test', (){
          blocks((){
            B.memory('mem1', 0, 2);
            B.instruction('inst1', 0, 3);
            B.subHakkPort('shp1', 0, 4, 'inst1', 'sh1', (){
              B.instruction('foo', 1, 2);
              B.shpBlock('mem1', 0, 1);
            });
          });
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
            codeBlock('inst1', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('nested SHP throws', (){
        doit() => hakk('test', (){
          blocks((){
            B.memory('mem1', 0, 2);
            B.instruction('inst1', 0, 3);
            B.subHakkPort('shp1', 0, 4, 'inst1', 'sh1', (){
              B.shpBlock('inst1', 0, 1);
              B.shpBlock('mem1', 0, 1);
              B.subHakkPort('shp1', 0, 4, 'inst1', 'sh1', (){
                B.shpBlock('inst1', 0, 1);
                B.shpBlock('mem1', 0, 1);
              });
            });
          });
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
            codeBlock('inst1', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('block name in shpBlock not found throws', (){
        doit() => hakk('test', (){
          blocks((){
            B.memory('mem1', 0, 2);
            B.instruction('inst1', 0, 3);
            B.subHakkPort('shp1', 0, 4, 'inst1', 'sh1', (){
              B.shpBlock('foo', 0, 1);
            });
          });
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
            codeBlock('inst1', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });

      test('sub-hakk block not yet declared throws', (){
        doit() => hakk('test', (){
          blocks((){
            B.instruction('inst1', 0, 3);
            B.subHakkPort('shp1', 0, 4, 'inst1', 'sh1', (){
              B.shpBlock('mem1', 0, 1);
            });
            B.memory('mem1', 0, 2);
          });
          code((){
            codeBlock('core', (){
              I.exit(0);
            });
            codeBlock('inst1', (){
              I.exit(0);
            });
          });
        });
        expect(doit, throwsA(new isInstanceOf<String>()));
      });
    
  });
}